import 'package:camerakit/CameraKitController.dart';
import 'package:camerakit/CameraKitView.dart';
import 'package:flutter/cupertino.dart';

// ignore: must_be_immutable
class Camera extends CameraKitView {
  Camera({
    Key key,
    bool useCamera2API = false,
    @required ScannerController bus,
    CameraType type = CameraType.back,
    CameraFlashState flashMode = CameraFlashState.auto,
    Function() onPermissionDenied,
    CameraScaleType scaleType = CameraScaleType.fill,
  }) : super(
          key: key,
          androidCameraMode: useCamera2API ? AndroidCameraMode.API_2 : AndroidCameraMode.API_X,
          hasBarcodeReader: false,
          cameraKitController: bus,
          cameraSelector: type.get,
          previewFlashMode: flashMode.get,
          onPermissionDenied: onPermissionDenied,
          scaleType: scaleType.get,
        );

  Camera.scanner({
    Key key,
    bool useCamera2API = true,
    @required ScannerController controller,
    CameraType type = CameraType.back,
    CameraFlashState flashMode = CameraFlashState.auto,
    Function() onPermissionDenied,
    Function(String data) onScanComplete,
    CameraScaleType scaleType = CameraScaleType.fill,
    CameraCode barcodeFormat = CameraCode.allFormats,
  }) : super(
          key: key,
          androidCameraMode: useCamera2API ? AndroidCameraMode.API_2 : AndroidCameraMode.API_2,
          hasBarcodeReader: true,
          cameraKitController: controller,
          cameraSelector: type.get,
          previewFlashMode: flashMode.get,
          onPermissionDenied: onPermissionDenied,
          onBarcodeRead: onScanComplete,
          barcodeFormat: barcodeFormat.get,
          scaleType: scaleType.get,
        );
}

class ScannerController extends CameraKitController {
  ScannerController() : super();

  setCameraFlash(CameraFlashState flash) => changeFlashMode(flash.get);
}

enum CameraType { front, back }

extension on CameraType {
  CameraSelector get get {
    switch (this) {
      case CameraType.back:
        return CameraSelector.back;
      case CameraType.front:
      default:
        return CameraSelector.front;
    }
  }
}

enum CameraScaleType { fit, fill }

extension on CameraScaleType {
  ScaleTypeMode get get {
    switch (this) {
      case CameraScaleType.fill:
        return ScaleTypeMode.fill;
      default:
        return ScaleTypeMode.fit;
    }
  }
}

enum CameraFlashState { on, off, auto }

extension on CameraFlashState {
  CameraFlashMode get get {
    switch (this) {
      case CameraFlashState.off:
        return CameraFlashMode.off;
      case CameraFlashState.on:
        return CameraFlashMode.on;
      default:
        return CameraFlashMode.auto;
    }
  }
}

enum CameraCode { allFormats, code128, code39, code93, codabar, dataMatrix, ean13, ean8, itf, qrCode, upcA, upcE, pdf417, asztec }

extension on CameraCode {
  BarcodeFormats get get {
    switch (this) {
      case CameraCode.allFormats:
        return BarcodeFormats.FORMAT_ALL_FORMATS;
      case CameraCode.asztec:
        return BarcodeFormats.FORMAT_AZTEC;
      case CameraCode.codabar:
        return BarcodeFormats.FORMAT_CODABAR;
      case CameraCode.code39:
        return BarcodeFormats.FORMAT_CODE_39;
      case CameraCode.code93:
        return BarcodeFormats.FORMAT_CODE_93;
      case CameraCode.code128:
        return BarcodeFormats.FORMAT_CODE_128;
      case CameraCode.dataMatrix:
        return BarcodeFormats.FORMAT_DATA_MATRIX;
      case CameraCode.ean8:
        return BarcodeFormats.FORMAT_EAN_8;
      case CameraCode.ean13:
        return BarcodeFormats.FORMAT_EAN_13;
      case CameraCode.itf:
        return BarcodeFormats.FORMAT_ITF;
      case CameraCode.pdf417:
        return BarcodeFormats.FORMAT_PDF417;
      case CameraCode.qrCode:
        return BarcodeFormats.FORMAT_QR_CODE;
      case CameraCode.upcA:
        return BarcodeFormats.FORMAT_UPC_A;
      case CameraCode.upcE:
        return BarcodeFormats.FORMAT_UPC_E;
      default:
        return BarcodeFormats.FORMAT_ALL_FORMATS;
    }
  }
}
