import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nueli/nueli.dart';

class NueliLogo extends Struct with Anim, Stance {
  final double density;
  final TranxConfig tranx;
  final Duration duration;
  final bool enforce;
  final bool reversed;
  final bool looped;

  NueliLogo({
    Key key,
    this.density = 2,
    this.tranx,
    this.duration,
    this.enforce = false,
    this.reversed = false,
    this.looped = false,
  });

  NueliLogo.sized({
    Key key,
    double size = 50,
    this.tranx,
    this.duration,
    this.enforce = false,
    this.reversed = false,
    this.looped = false,
  }) : this.density = size / 50;

  @override
  Widget get body {
    stance = [enforce, reversed, looped, density, tranx];
    return Transform.translate(
      offset: Offset(-3 * density.toDouble(), 0),
      child: AnimatedBuilder(
        animation: animation.drive(Tween<double>(begin: 0, end: 1)),
        builder: (context, child) {
          return CustomPaint(
            size: Size.square((this.density * 50).toDouble()),
            painter: _AnimatedLogoPainter()
              .._density = ((density * 6) + 0.1993 * density)
              .._value = animation.value,
          );
        },
      ),
    );
  }

  @override
  void stanceAction() => looped ? repeat(reverse: reversed, period: duration) : forward();
}

class _AnimatedLogoPainter extends CustomPainter {
  double _density;
  double _value;

  @override
  void paint(Canvas canvas, Size size) {
    var paint = Paint()
      ..color = palette.nueliGrey
      ..style = PaintingStyle.stroke
      ..strokeCap = StrokeCap.square;

    canvas.drawCircle(Offset(size.width * 4 / 5 - _density * 3.98 / 12, size.height - _density * 2.5 / 2.89), _value * _density * 3.5 / 4.5, Paint()..color = palette.nueliOrange);

    canvas.clipRect(Rect.fromLTWH(size.width / 5, -size.height / 2, size.width * 4 / 5, size.height * 1.5 - 2), doAntiAlias: true);

    canvas.clipPath(
        Path()
          ..fillType = PathFillType.evenOdd
          ..moveTo(size.width * 1 / 5 + _density / 4, size.height)
          ..relativeLineTo(0, -size.height * 0.8 - _density / 4)
          ..relativeLineTo(size.width * 3 / 5 + _density / 2, -size.width * 0.2 - _density / 4)
          ..lineTo(size.width * 4 / 5 + _density * 3 / 4, size.height)
          ..addPath(
            Path()..relativeArcToPoint(Offset(_density * 3.5 / 1.63, 0), radius: Radius.circular(_density * 3.5 / 16)),
            Offset(size.width * 4 / 5 - _density * 3.5 / 2.5, size.height - _density * 2.5 / 2.3),
          )
          ..relativeLineTo(0, _density * 3.8)
          ..relativeLineTo(-_density * 3.8, 0),
        doAntiAlias: true);
    canvas.drawPath(
        (Path()
          ..fillType = PathFillType.evenOdd
          ..moveTo((size.width * 1 / 5 + _density / 2), size.height)
          ..relativeLineTo(0, _value * 5 * (-size.height * 0.8 + _density / 2))),
        paint..strokeWidth = _density * 3.5);
    canvas.drawPath(
        _value > 2 / 5
            ? ((Path()
              ..moveTo(size.width * 4 / 5 + _density / 2.6, 0)
              ..relativeLineTo(0, ((_value - 2 / 5) * 5 * (size.height - _density * 0.9)))))
            : Path(),
        paint..strokeWidth = _density * 3.5);
    canvas.drawPath(
      _value > 1 / 5
          ? (Path()
            ..moveTo(size.width * 1 / 5 + _density / 1.3, size.height * 0.2)
            ..relativeLineTo(
              (_value - 1 / 5) * 5 * size.width * 3 / 5,
              (_value - 1 / 5) * 5 * -size.width * 0.2,
            ))
          : Path(),
      paint..strokeWidth = _density,
    );
  }

  bool shouldRepaint(covariant CustomPainter oldDelegate) => true;
}
