import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nueli/nueli.dart';

/// Pager2
class Pager2 extends Struct {
  final Ref<TabController> _tabController = $();
  final Ref<PageController> _pageController = $();

  final Widget generalBackground;
  final List<Page2> sections;
  final PageController controller;
  final Color indicatorColor;
  final double indicatorHeight;
  final tabBarHeight;
  final Duration duration;
  final Curve curve;

  Pager2({
    this.sections = const [],
    @required this.controller,
    this.generalBackground,
    this.indicatorColor,
    this.indicatorHeight = 3,
    this.tabBarHeight = 40,
    this.duration = const Duration(milliseconds: 500),
    this.curve = Curves.linearToEaseOut,
  });

  geoToSection(int section, {Duration duration, Curve curve}) {
    return _pageController.data.animateToPage(
      section,
      duration: duration ?? this.duration,
      curve: curve ?? this.curve,
    );
  }

  Widget _tabs(BuildContext context) {
    final tabs = sections.map((e) => Tab(child: e.secondTitle(context))).toList();
    return SizedBox(
      height: tabBarHeight,
      child: TabBar(
        tabs: tabs,
        onTap: (i) => geoToSection(i),
        controller: _tabController.data,
        physics: AlwaysScrollableScrollPhysics(parent: BouncingScrollPhysics()),
        isScrollable: sections.length > 3,
        indicatorSize: TabBarIndicatorSize.label,
        automaticIndicatorColorAdjustment: true,
        indicatorWeight: indicatorHeight,
        labelPadding: EdgeInsets.symmetric(vertical: 8),
        overlayColor: MaterialStateProperty.all<Color>(palette.accent.withAlpha(50)),
        indicatorColor: indicatorColor ?? palette.accent,
        indicator: [
          $decor.paint((indicatorColor ?? palette.accent).withAlpha(25)),
          $decor.stroke(0.5, (indicatorColor ?? palette.accent).withAlpha(100)),
        ].foldThrough().compile,
      ),
    );
  }

  Widget _pageView(BuildContext context) {
    return PageView(
      children: sections.map((e) => e.sectionContent(context)).toList(),
      controller: _pageController.data,
      allowImplicitScrolling: true,
      onPageChanged: (i) => _tabController.data.animateTo(i, curve: curve, duration: duration),
      physics: AlwaysScrollableScrollPhysics(parent: BouncingScrollPhysics()),
      clipBehavior: Clip.none,
    );
  }

  @override
  Widget get body => Stack(children: [
        generalBackground,
        _pageView(context),
        _tabs(context),
      ]);

  @override
  StructState<Struct> createState() => _Pager2State();
}

class _Pager2State<T extends Pager2> extends StructState<T> {
  @override
  void dispose() {
    widget._pageController.data.dispose();
    super.dispose();
  }
}

class Page2 with UI {
  final Widget Function(BuildContext, Widget child) contentBuilder;
  final Widget Function(BuildContext, Widget child) titleBuilder;
  final Widget content;
  final Widget title;

  Widget sectionContent(BuildContext context) => contentBuilder != null
      ? contentBuilder(context, content ?? gap(0)) // break
      : content ?? gap(0);

  Widget secondTitle(BuildContext context) => titleBuilder != null
      ? titleBuilder(context, title ?? gap(0)) // break
      : title ?? gap(0);

  Page2({
    this.contentBuilder,
    this.titleBuilder,
    this.content = const SizedBox(),
    Widget title = const SizedBox(),
  }) : title = Padding(
          padding: EdgeInsets.symmetric(horizontal: 8),
          child: title,
        );

  Page2.withTitle({
    String title,
    Color titleColor,
    Widget Function(BuildContext context, Widget) builder,
    this.content,
  })  : contentBuilder = builder,
        titleBuilder = null,
        title = Padding(
          padding: EdgeInsets.symmetric(horizontal: 8),
          child: $text.color(titleColor ?? palette.accent).align_center.build(data: title),
        );
}
