import 'package:flutter/material.dart';
import 'package:nueli/nueli.dart';

import '../components/camera.dart';

class ScannerDialog extends Screen {
  final Color borderColor;
  final Color laserColor;
  final Function(String data) onScanComplete;
  final Function() onScanFailed;

  @override
  _ScannerDialogState createState() => _ScannerDialogState();

  ScannerDialog.show(
    BuildContext context, {
    @required this.onScanComplete,
    this.onScanFailed,
    this.borderColor,
    this.laserColor,
    bool replace = false,
    bool closeAllPrev = false,
  })  : assert(onScanComplete != null),
        super.open(context, closeAllPrev: closeAllPrev, replace: replace);
}

class _ScannerDialogState extends ScreenState<ScannerDialog> with Fragment<ScannerDialog> {
  ScannerController _controller;
  AnimationController _shadeController;
  bool _toggleFlash = true;

  Future<void> initData() async {
    _controller = ScannerController();
    _shadeController = AnimationController(vsync: this, duration: Duration(seconds: 1));
    _shadeController..repeat(reverse: true);
    Future.delayed(Duration(seconds: 1), () => _controller.resumeCamera());
  }

  @override
  void init() {
    initData();
    super.init();
  }

  get tertiaryTranx => $tranx.fade(0, 1).pan_vert(1, 0).scale(1.2, 1);

  @override
  bool get delayTertiary => true;

  @override
  Widget get body {
    return safeRegion(
      withSafeArea: true,
      child: Stack(
        children: [
          GestureDetector(onTap: close),
          Center(child: transform),
          Center(
            child: Container(
              alignment: Alignment.bottomRight,
              height: MediaQuery.of(context).size.width / 1.5,
              width: MediaQuery.of(context).size.width / 1.2,
              child: IconButton(
                  icon: !_toggleFlash ? $icon.color_bg.flash : $icon.color(palette.text).flashOff,
                  onPressed: () async {
                    _controller.setCameraFlash(_toggleFlash ? CameraFlashState.off : CameraFlashState.on);
                    setState(() => _toggleFlash = !_toggleFlash);
                  }),
            ),
          ),
        ],
      ),
    );
  }

  Widget get transform {
    return $tranx.fade(1, 0).build(
          data: animation,
          child: Container(
            clipBehavior: Clip.antiAliasWithSaveLayer,
            padding: EdgeInsets.all(5),
            decoration: $decor.paint_bg.radii_16.raise(24, palette.text.withAlpha(150)).compile,
            child: GestureDetector(
              onTap: () => _controller.resumeCamera(),
              child: Container(
                clipBehavior: Clip.antiAliasWithSaveLayer,
                decoration: $decor.paint_bg.radii_16.compile,
                height: MediaQuery.of(context).size.width / 1.5,
                width: MediaQuery.of(context).size.width / 1.2,
                child: Stack(fit: StackFit.expand, children: [
                  Camera.scanner(
                    controller: _controller,
                    flashMode: CameraFlashState.auto,
                    scaleType: CameraScaleType.fill,
                    onScanComplete: (data) {
                      Screens.goBack(context);
                      if (data == null && widget.onScanFailed != null) widget.onScanFailed();
                      if (data != null) widget.onScanComplete(data);
                    },
                  ),
                  AlignTransition(
                    alignment: AlignmentTween(
                      begin: Alignment(-2, 0),
                      end: Alignment(2, 0),
                    ).animate(_shadeController),
                    child: $tranx.fade(1, 0).build(
                          data: _shadeController,
                          child: Container(
                            width: 150,
                            decoration: $decor.linear
                                .paint((widget.laserColor ?? palette.accent).withAlpha(50)) // break
                                .paint((widget.laserColor ?? palette.accent).withAlpha(150))
                                .paint((widget.laserColor ?? palette.accent))
                                .paint((widget.laserColor ?? palette.accent).withAlpha(150))
                                .paint((widget.laserColor ?? palette.accent).withAlpha(0))
                                .compile,
                          ),
                        ),
                  ),
                ]),
              ),
            ),
          ),
        );
  }

  @override
  void dispose() {
    _shadeController.dispose();
    super.dispose();
  }
}
